using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.IO;
using System.Runtime.InteropServices;

namespace SpringClient
{
	/// <summary>
	/// Summary description for MapSelector.
	/// </summary>
	public class MapSelector : System.Windows.Forms.Form
	{
		[DllImport(@".\Utility.dll")]
		private static extern bool GetMapinfo(string filename, ref SMapInfo mi);

		[StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
		public struct SMapInfo
		{
			public String Name;
			public int TidalStrength;
			public int Gravity;
			public float MaxMetal;
			public int ExtractorRadius;
			public int MinWind;
			public int MaxWind;
		}

		private System.Windows.Forms.ListBox MapsListbox;
		private ClientControls.Minimap		Minimap;
		private Label						NameLabel;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.Label label1;
		private System.Windows.Forms.Label label2;
		private System.Windows.Forms.Label label3;
		private System.Windows.Forms.Label label4;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label LMapName;
		private System.Windows.Forms.Label LTidal;
		private System.Windows.Forms.Label LGravity;
		private System.Windows.Forms.Label LMaxMetal;
		private System.Windows.Forms.Label LExtractor;
		private System.Windows.Forms.Label LMinWind;
		private System.Windows.Forms.Label LMaxWind;
		private SMapInfo		m_MapInfo;

		public SMapInfo MapInfo
		{
			get
			{
				return m_MapInfo;
			}
		}

		public string Mapfilename
		{
			get
			{
				return MapsListbox.Text;
			}
		}

		public string Mapname
		{
			get
			{
				string ret = MapsListbox.Text.Substring(MapsListbox.Text.LastIndexOf("\\".ToString()));
				return ret.Substring(1, ret.Length-5);
			}
		}

		public MapSelector(ClientControls.Minimap minimap, Label namelabel)
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			Minimap = minimap;
			NameLabel = namelabel;
			//
			// TODO: Add any constructor code after InitializeComponent call
			//

			string[] files = Directory.GetFiles(string.Format(@"{0}\maps", Global.SpringDirectory), "*.smd");
			foreach ( string file in files )
			{
				MapsListbox.Items.Add(file);
			}

			if ( files.Length > 0 )
				MapsListbox.SelectedIndex = 0;
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MapSelector));
			this.MapsListbox = new System.Windows.Forms.ListBox();
			this.label1 = new System.Windows.Forms.Label();
			this.label2 = new System.Windows.Forms.Label();
			this.label3 = new System.Windows.Forms.Label();
			this.label4 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label7 = new System.Windows.Forms.Label();
			this.LMapName = new System.Windows.Forms.Label();
			this.LTidal = new System.Windows.Forms.Label();
			this.LGravity = new System.Windows.Forms.Label();
			this.LMaxMetal = new System.Windows.Forms.Label();
			this.LExtractor = new System.Windows.Forms.Label();
			this.LMinWind = new System.Windows.Forms.Label();
			this.LMaxWind = new System.Windows.Forms.Label();
			this.SuspendLayout();
			// 
			// MapsListbox
			// 
			this.MapsListbox.Location = new System.Drawing.Point(16, 16);
			this.MapsListbox.Name = "MapsListbox";
			this.MapsListbox.Size = new System.Drawing.Size(240, 108);
			this.MapsListbox.TabIndex = 0;
			this.MapsListbox.SelectedIndexChanged += new System.EventHandler(this.MapsListbox_SelectedIndexChanged);
			// 
			// label1
			// 
			this.label1.Location = new System.Drawing.Point(16, 144);
			this.label1.Name = "label1";
			this.label1.Size = new System.Drawing.Size(104, 16);
			this.label1.TabIndex = 1;
			this.label1.Text = "Description";
			// 
			// label2
			// 
			this.label2.Location = new System.Drawing.Point(16, 184);
			this.label2.Name = "label2";
			this.label2.Size = new System.Drawing.Size(104, 16);
			this.label2.TabIndex = 2;
			this.label2.Text = "Tidal Strength";
			// 
			// label3
			// 
			this.label3.Location = new System.Drawing.Point(16, 216);
			this.label3.Name = "label3";
			this.label3.Size = new System.Drawing.Size(104, 16);
			this.label3.TabIndex = 3;
			this.label3.Text = "Gravity";
			// 
			// label4
			// 
			this.label4.Location = new System.Drawing.Point(16, 248);
			this.label4.Name = "label4";
			this.label4.Size = new System.Drawing.Size(104, 16);
			this.label4.TabIndex = 4;
			this.label4.Text = "Max Metal";
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(16, 280);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(104, 16);
			this.label5.TabIndex = 5;
			this.label5.Text = "Extractor Radius";
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(16, 312);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(104, 16);
			this.label6.TabIndex = 6;
			this.label6.Text = "Min Wind";
			// 
			// label7
			// 
			this.label7.Location = new System.Drawing.Point(16, 344);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(104, 16);
			this.label7.TabIndex = 7;
			this.label7.Text = "Max Wind";
			// 
			// LMapName
			// 
			this.LMapName.Location = new System.Drawing.Point(144, 136);
			this.LMapName.Name = "LMapName";
			this.LMapName.Size = new System.Drawing.Size(104, 40);
			this.LMapName.TabIndex = 8;
			// 
			// LTidal
			// 
			this.LTidal.Location = new System.Drawing.Point(144, 184);
			this.LTidal.Name = "LTidal";
			this.LTidal.Size = new System.Drawing.Size(104, 16);
			this.LTidal.TabIndex = 9;
			// 
			// LGravity
			// 
			this.LGravity.Location = new System.Drawing.Point(144, 216);
			this.LGravity.Name = "LGravity";
			this.LGravity.Size = new System.Drawing.Size(104, 16);
			this.LGravity.TabIndex = 10;
			// 
			// LMaxMetal
			// 
			this.LMaxMetal.Location = new System.Drawing.Point(144, 248);
			this.LMaxMetal.Name = "LMaxMetal";
			this.LMaxMetal.Size = new System.Drawing.Size(104, 16);
			this.LMaxMetal.TabIndex = 11;
			// 
			// LExtractor
			// 
			this.LExtractor.Location = new System.Drawing.Point(144, 280);
			this.LExtractor.Name = "LExtractor";
			this.LExtractor.Size = new System.Drawing.Size(104, 16);
			this.LExtractor.TabIndex = 12;
			// 
			// LMinWind
			// 
			this.LMinWind.Location = new System.Drawing.Point(144, 312);
			this.LMinWind.Name = "LMinWind";
			this.LMinWind.Size = new System.Drawing.Size(104, 16);
			this.LMinWind.TabIndex = 13;
			// 
			// LMaxWind
			// 
			this.LMaxWind.Location = new System.Drawing.Point(144, 344);
			this.LMaxWind.Name = "LMaxWind";
			this.LMaxWind.Size = new System.Drawing.Size(104, 16);
			this.LMaxWind.TabIndex = 14;
			// 
			// MapSelector
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(272, 374);
			this.Controls.Add(this.LMaxWind);
			this.Controls.Add(this.LMinWind);
			this.Controls.Add(this.LExtractor);
			this.Controls.Add(this.LMaxMetal);
			this.Controls.Add(this.LGravity);
			this.Controls.Add(this.LTidal);
			this.Controls.Add(this.LMapName);
			this.Controls.Add(this.label7);
			this.Controls.Add(this.label6);
			this.Controls.Add(this.label5);
			this.Controls.Add(this.label4);
			this.Controls.Add(this.label3);
			this.Controls.Add(this.label2);
			this.Controls.Add(this.label1);
			this.Controls.Add(this.MapsListbox);
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Name = "MapSelector";
			this.Text = "MapSelector";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MapSelector_Closing);
			this.ResumeLayout(false);

		}
		#endregion

		private void MapsListbox_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			try
			{
				ListBox lb = (ListBox)sender;
				m_MapInfo = new SMapInfo();
				m_MapInfo.Name = new String('\0', 255);
				string filename = lb.Text;
				string mapfile = filename.Replace(".smd", ".sm2");
				if ( !GetMapinfo(filename, ref m_MapInfo) )
				{
					MessageBox.Show("Failed to get map info");
				}
				else
				{
					NameLabel.Text = Mapname;
					LMapName.Text = m_MapInfo.Name.ToString();
					LTidal.Text = m_MapInfo.TidalStrength.ToString();
					LGravity.Text = m_MapInfo.Gravity.ToString();
					LMaxMetal.Text = m_MapInfo.MaxMetal.ToString();
					LExtractor.Text = m_MapInfo.ExtractorRadius.ToString();
					LMinWind.Text = m_MapInfo.MinWind.ToString();
					LMaxWind.Text = m_MapInfo.MaxWind.ToString();
				}
				Minimap.CloseImage();
				Minimap.Open(mapfile);
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}

		private void MapSelector_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			try
			{
				// don't let the form close.  cancel the close and just hide it
				e.Cancel = true;
				Hide();
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
			}
		}
	}
}
