#pragma once

#include "float3.h"
#include "command.h"
struct UnitDef;

typedef void (* ENTERLOSCALLBACK)(int spotter, int spotted);

class IGlobalAICallback
{
public:
	virtual void InitCallback(int team)=0;
	// not sure how this will work yet, as in IGlobalAI
	virtual void SendCommandMsg(int team)=0;

	// returns the size of the created area
	virtual int CreateSharedArea(char* name, int size);

	// if null is returned, no shared area exists
	// returns the size of the area through the size parameters
	virtual void* GetSharedArea(char* name, int* size);

	// taunting :-D
	virtual void SendTextMsg(int player, const char* text, int priority)=0;
	
	virtual int GetCurrentFrame()=0;

	virtual void AddEnterLosCallback(int unit, ENTERLOSCALLBACK cback);
	virtual void DeleteEnterLosCallback(int unit)=0;

	virtual int GiveOrder(int unitid, Command* c)=0;
	// this sends a group order to what the AI has "selected".
	// must have called SendSelection
	virtual int GiveGroupOrder(Command* c)=0;

	virtual Command* GetCurrentCommand(int unitid)=0;
	// returns the number of commands in the queue for a given unit
	virtual int GetCommandQueueLen(int unitid)=0;
	// returns the command queue for a particular unit.  up to the AI to allocate sufficient
	// memory for the call, which is sent in the memsize parameter.  returns -1 on fail due to insufficient memory
	virtual int GetCommandQueue(int unitid, Command* commands, int memsize)=0;

	// use this to sync up a selected group, if you want to send many mass orders
	// to the same group of units.  most AIs probably won't use this
	virtual int SendSelection(const int* units, int numunits)=0;
	virtual int GetSelection(int* units, int memsize)=0;

	// as per the Group AI, these return 0 if you don't have los to the unit in question
	virtual int GetUnitAiHint(int unitid)=0;
	virtual int GetUnitTeam(int unitid)=0;
	virtual float GetUnitHealth(int unitid)=0;			//the units current health
	virtual float GetUnitMaxHealth(int unitid)=0;		//the units max health
	virtual float GetUnitSpeed(int unitid)=0;				//the units max speed
	virtual float GetUnitPower(int unitid)=0;				//sort of the measure of the units overall power
	virtual float GetUnitExperience(int unitid)=0;	//how experienced the unit is (0.0-1.0)
	virtual float GetUnitMaxRange(int unitid)=0;		//the furthest any weapon of the unit can fire
	virtual const UnitDef* GetUnitDef(int unitid)=0;	//this returns the units unitdef struct from which you can read all the statistics of the unit, dont try to change any values in it, dont use this if you dont have to risk of changes in it

	virtual const UnitDef* GetUnitDef(const char* unitName)=0;

	//this will return a value even if you only have radar to it, but it might be wrong
	virtual float3 GetUnitPos(int unitid)=0;				//note that x and z are the horizontal axises while y represent height


	//the following functions allows the dll to use the built in pathfinder
	//call InitPath and you get a pathid back
	//use this to call GetNextWaypoint to get subsequent waypoints, the waypoints are centered on 8*8 squares
	//note that the pathfinder calculates the waypoints as needed so dont retrieve them until they are needed
	//the waypoints x and z coordinate is returned in x and z while y is used for error codes
	//>=0 =worked ok,-2=still thinking call again,-1= end of path reached or invalid path
	virtual int InitPath(float3 start,float3 end,int pathType)=0;
	virtual float3 GetNextWaypoint(int pathid)=0;
	virtual void FreePath(int pathid)=0;

	//This function returns the approximate path cost between two points(note that it needs to calculate the complete path so its somewhat expansive)
	virtual float GetPathLength(float3 start,float3 end,int pathType)=0;

	//the following function return the units into arrays that must be allocated by the dll
	//10000 is currently the max amount of units so that should be a safe size for the array
	//the return value indicates how many units was returned, the rest of the array is unchanged
	virtual int GetEnemyUnits(int *units)=0;					//returns all known enemy units
	virtual int GetEnemyUnits(int *units,const float3& pos,float radius)=0; //returns all known enemy units within radius from pos
	virtual int GetFriendlyUnits(int *units)=0;					//returns all friendly units
	virtual int GetFriendlyUnits(int *units,const float3& pos,float radius)=0; //returns all friendly units within radius from pos

	//the following functions are used to get information about the map
	//dont modify or delete any of the pointers returned
	//the maps are stored from top left and each data position is 8*8 in size
	//to get info about a position x,y look at location
	//(int(y/8))*GetMapWidth()+(int(x/8))
	//some of the maps are stored in a lower resolution than this though
	virtual int GetMapWidth()=0;
	virtual int GetMapHeight()=0;
	virtual const float* GetHeightMap()=0;						//this is the height for the center of the squares, this differs slightly from the drawn map since it uses the height at the corners
	virtual const unsigned short* GetLosMap()=0;			//a square with value zero means you dont have los to the square, this is half the resolution of the standard map
	virtual const unsigned short* GetRadarMap()=0;		//a square with value zero means you dont have radar to the square, this is 1/8 the resolution of the standard map
	virtual const unsigned short* GetJammerMap()=0;		//a square with value zero means you dont have radar jamming on the square, this is 1/8 the resolution of the standard map
	virtual const unsigned char* GetMetalMap()=0;			//this map shows the metal density on the map, this is half the resolution of the standard map

	virtual float GetElevation(float x,float z)=0;		//Gets the elevation of the map at position x,z

	virtual bool CanBuildAt(const UnitDef* unitDef,float3 pos)=0; //returns true if a given type of unit can be build at a pos (not blocked by other units etc)

	virtual float GetMetal()=0;				//stored metal for team
	virtual float GetMetalRate()=0;				// rate flow for metal
	virtual float GetEnergy()=0;				//stored energy for team
	virtual float GetEnergyRate()=0;				// rate flow for metal
	virtual float GetMetalStorage()=0;				//metal storage for team
	virtual float GetEnergyStorage()=0;				//energy storage for team
};